import pytest
from fastapi import FastAPI, Request
from fastapi.testclient import TestClient
from starlette.responses import PlainTextResponse
from response_bandwidth_limiter import ResponseBandwidthLimiter, ResponseBandwidthLimitExceeded, _response_bandwidth_limit_exceeded_handler
import time

# デコレータAPIのテスト
def test_limiter_decorator():
    app = FastAPI()
    limiter = ResponseBandwidthLimiter()
    app.state.response_bandwidth_limiter = limiter
    app.add_exception_handler(ResponseBandwidthLimitExceeded, _response_bandwidth_limit_exceeded_handler)
    
    # 帯域制限付きエンドポイント (200 bytes/sec)
    @app.get("/test")
    @limiter.limit(200)
    async def read_test(request: Request):
        return PlainTextResponse("a" * 600)
    
    client = TestClient(app)
    
    # ルート登録の検証
    assert "read_test" in limiter.routes
    assert limiter.routes["read_test"] == 200
    
    # レスポンス内容の検証
    response = client.get("/test")
    assert response.status_code == 200
    assert len(response.content) == 600
    
    # 注: テスト環境ではasyncioのsleepが適切に機能しないため
    # 時間計測による検証はスキップします

# 不正な引数のテスト
def test_invalid_limit_argument():
    limiter = ResponseBandwidthLimiter()
    
    # 文字列を渡すと例外が発生する
    with pytest.raises(TypeError):
        @limiter.limit("not_a_number")
        async def invalid_test(request: Request):
            pass
    
    # 正しく動作する整数の場合
    @limiter.limit(1000)
    async def valid_test(request: Request):
        pass
    
    # ルート名が正しく保存されているか
    assert "valid_test" in limiter.routes
    assert limiter.routes["valid_test"] == 1000
